//----------------------------------------------------------------------------
// C main line
//----------------------------------------------------------------------------

#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules

/* ---------------- */
/* --- global       */
/* ---------------- */

/* ----- LED color (0-63) */
BYTE  gLedRed, gLedGreen, gLedBlue;
/* --- rainbow seq */
BYTE  gRainbowStep;
/* --- charactor color */
BYTE  gCharactor;

/* --- wait work */
BYTE  gWaitCounter;
BYTE  gWaitSync;

/* --- flash mode */
BYTE  gFlashMode;
BYTE  gFlashCount;


/* --------------------------------------- */
/* --- interrupt                           */
/* --------------------------------------- */
#pragma interrupt_handler Timer1Int
void Timer1Int(void)
{
	gWaitSync = 0;
	if (gWaitCounter > 0) {
		gWaitCounter--;
	}
	if (gFlashMode == 1) {
		gFlashCount++;
		if (gFlashCount > 30) {
			gFlashCount = 0;
		}
		if (gFlashCount > 20) {
			PRT0GS = 0b00000100;
			PRT1GS = 0b00000011;
		}
		else {
			PRT0GS = 0b00000000;
			PRT1GS = 0b00000000;
		}
	}
	else {
		PRT0GS = 0b00000100;
		PRT1GS = 0b00000011;
	}
}

void waitASync(void)
{
	gWaitSync = 1;
	while(gWaitSync == 1) {}
}

/* --------------------------------------- */
/* --- LED control                         */
/* --------------------------------------- */

void setPwm(void)
{
	BYTE  ch1, ch2, ch3;
	WORD  l1, l2, l3;
	l1 = 0x52 * gLedRed;
	l2 = 0xfe * gLedGreen;
	l3 = 0xb8 * gLedBlue;
	ch1 = l1 >> 6;
	ch2 = l2 >> 6;
	ch3 = l3 >> 6;
	PWM8_1_WritePulseWidth(0xff - ch1);
	PWM8_2_WritePulseWidth(0xff - ch3);
	PWM8_3_WritePulseWidth(0xff - ch2);
}

void setCharactorColor(BYTE index)
{
	switch(index) {
		case 1:
			gLedRed = 63;
			gLedGreen = 0;
			gLedBlue = 0;
			gRainbowStep = 1;
			break;

		case 2:
			gLedRed = 63;
			gLedGreen = 31;
			gLedBlue = 0;
			gRainbowStep = 1;
			break;

		case 3:
			gLedRed = 63;
			gLedGreen = 50;
			gLedBlue = 0;
			gRainbowStep = 1;
			break;

		case 4:
			gLedRed = 40;
			gLedGreen = 63;
			gLedBlue = 0;
			gRainbowStep = 2;
			break;

		case 5:
			gLedRed = 0;
			gLedGreen = 63;
			gLedBlue = 21;
			gRainbowStep = 3;
			break;

		case 6:
			gLedRed = 0;
			gLedGreen = 0;
			gLedBlue = 63;
			gRainbowStep = 4;
			break;

		case 7:
			gLedRed = 63;
			gLedGreen = 0;
			gLedBlue = 31;
			gRainbowStep = 0;
			break;


		default:
			gLedRed = 63;
			gLedGreen = 63;
			gLedBlue = 63;
			gRainbowStep = 1;
			break;
	}

	setPwm();
}

/* --------------------------------------- */
/* --- rainbow work                        */
/* --------------------------------------- */

void doRainbowStep(void)
{
	switch(gRainbowStep) {
		case 0:
			if (gLedGreen > 0) { gLedGreen--; }
			if (gLedBlue > 0) { gLedBlue--; }
			if ((gLedGreen == 0) && (gLedBlue == 0)) { gRainbowStep = 1; }
			break;
		
		case 1:
			if (gLedRed < 63) { gLedRed++; }
			if (gLedGreen < 63) { gLedGreen++; }
			if ((gLedRed == 63) && (gLedGreen == 63)) { gRainbowStep = 2; }
			break;
		
		case 2:
			if (gLedRed > 0) { gLedRed--; }
			if (gLedBlue > 0) { gLedBlue--; }
			if ((gLedRed == 0) && (gLedBlue == 0)) { gRainbowStep = 3; }
			break;
		
		case 3:
			if (gLedGreen < 63) { gLedGreen++; }
			if (gLedBlue < 63) { gLedBlue++; }
			if ((gLedGreen == 63) && (gLedBlue == 63)) { gRainbowStep = 4; }
			break;

		case 4:
			if (gLedRed > 0) { gLedRed--; }
			if (gLedGreen > 0) { gLedGreen--; }
			if ((gLedRed == 0) && (gLedGreen == 0)) { gRainbowStep = 5; }
			break;
		
		case 5:
			if (gLedRed < 63) { gLedRed++; }
			if (gLedBlue < 63) { gLedBlue++; }
			if ((gLedRed == 63) && (gLedBlue == 63)) { gRainbowStep = 0; }
			break;

		default:
			gRainbowStep = 0;
			break;
	}
	
	setPwm();
}

/* --------------------------------------- */
/* --- main routine                        */
/* --------------------------------------- */
void main(void)
{
	BYTE  pin_in;

	gWaitCounter = 0;
	gWaitSync = 0;
	gFlashMode = 0;
	gFlashCount = 0;

	PRT0DR = 0x0e;
	PRT1DR = 0x03;
	pin_in = PRT0DR;
	pin_in &= 0b00001000;
	if (pin_in == 0) {
		gFlashMode = 1;
	}

    PWM8_1_Start();
    PWM8_2_Start();
    PWM8_3_Start();
	Counter16_1_EnableInt();
	M8C_EnableGInt;
	Counter16_1_Start();

	gCharactor = 0;
	gRainbowStep = 0;
	setCharactorColor(gCharactor);

	while(pin_in == 0) {
		pin_in = PRT0DR;
		pin_in &= 0b00001000;
	}

	while(1) {
		pin_in = PRT0DR;
		pin_in &= 0b00001000;
		if (pin_in == 0) {
			gWaitCounter = 60;
			while(pin_in == 0) {
				if (gWaitCounter == 0) {
					doRainbowStep();
				}
				waitASync();
				pin_in = PRT0DR;
				pin_in &= 0b00001000;
			}
			if (gWaitCounter > 0) {
				gCharactor = (gCharactor + 1) % 8;
				setCharactorColor(gCharactor);
			}
		}
		waitASync();
	}
}
